#! /usr/bin/perl
# squidGuardUpdate.cgi - Created by James Pattie (james@pcxperience.com)
# 11/06/2001  The script that the firewalls query to see if they have new squidGuard DB files to download.
# Copyright (c) 2000-2002, Xperience, Inc. (http://www.pcxperience.com/)
#
#This program is free software; you can redistribute it and/or
#modify it under the terms of the GNU General Public License
#as published by the Free Software Foundation; either version 2
#of the License, or (at your option) any later version.
#
#This program is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#GNU General Public License for more details.
#
#You should have received a copy of the GNU General Public License
#along with this program; if not, write to the Free Software
#Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

use strict;
use DBIWrapper;
use HTMLObject::Base;
use Portal::Data::Config;
use Portal::Auth;
use Portal::squidGuard::Data::Config;
use HTMLObject::CGILib;
use POSIX qw(tmpnam);

use vars qw( $portalDB );

# instantiate the Portal::Data::Config object.
my $configObj = undef;
eval { $configObj = Portal::Data::Config->new; };
if ($@)
{
  # hopefully this will never happen!
  myDie(error => "Error instantiating Portal::Data::Config->new()!  Error = $@\n");
}

# instantiate connections to the Portal database.
if ($configObj->dbType eq "Postgres")
{
  $portalDB = DBIWrapper->new(dbHost => $configObj->dbHost, dbName => $configObj->dbName, dbUser => $configObj->dbUser, dbPasswd => $configObj->dbPasswd, dbPort => $configObj->dbPort);
  if ($portalDB->didErrorOccur)
  {
    myDie(error => $portalDB->errorMessage);
  }
}
elsif ($configObj->dbType eq "MySQL")
{
  $portalDB = DBIWrapper->new(dbType => "mysql", dbHost => $configObj->dbHost, dbName => $configObj->dbName, dbUser => $configObj->dbUser, dbPasswd => $configObj->dbPasswd, dbPort => $configObj->dbPort);
  if ($portalDB->didErrorOccur)
  {
    myDie(error => $portalDB->errorMessage);
  }
}
else
{
  myDie(error => "dbType = '$configObj->{dbType}' is invalid!");
}

# parse apart the environment and get anything coming into the script.
use vars qw ( %input );
HTMLObject::CGILib::ReadParse(*input);

my $doc = undef;  # The HTMLObject that will be created.
$doc = HTMLObject::Base->new;
$doc->setContentType("application/x-gzip");  # we are a tarball!

# gather incoming variables we require.
my $companyCode = $input{companyCode};
my $companyId = -1;
my $serverName = $input{serverName};
my $serverId = -1;
my $realPass = "";
my $serverBLGroups = "";
my $serverPass = $input{serverPass};
my $serverVersion = $input{serverVersion};
my $dbVersion = 0;

if (length $companyCode == 0)
{
  myDie(error => "companyCode must be specified!");
}
if (length $serverName == 0)
{
  myDie(error => "serverName must be specified!");
}
if (length $serverPass < 6)
{
  myDie(error => "serverPass must be 6 characters or more!");
}
if ($serverVersion !~ /^(\d+)$/)
{
  myDie(error => "serverVersion = '$serverVersion' is invalid!");
}

# create the authObj for this session to work with.
my $authObj = Portal::Auth->new(portalDB => $portalDB);
if ($authObj->didErrorOccur)
{
  myDie(error => $authObj->errorMessage);
}

# lookup the company based upon the specified companyCode.
my $companyObj = $authObj->getCompanyInfo(code => $companyCode);
if (!defined $companyObj)
{
  if ($authObj->didErrorOccur)
  {
    myDie(error => $authObj->errorMessage);
  }
  else
  {
    myDie(error => "companyCode = '$companyCode' does not exist!");
  }
}
else
{
  if ($companyObj->didErrorOccur)
  {
    myDie(error => $companyObj->errorMessage);
  }
  else
  {
    $companyId = $companyObj->{id};
  }
}

# instantiate the squidGuard::Data::Config module to get the settings
my $squidGuardConf = undef;
eval { $squidGuardConf = Portal::squidGuard::Data::Config->new(); };
if ($@)
{
  myDie(error => $@);
}

# now create a connection to the squidguard_db database.
my $squidGuardDB = DBIWrapper->new(dbHost => $configObj->dbHost, dbName => $squidGuardConf->{dbName}, dbUser => $configObj->dbUser, dbPasswd => $configObj->dbPasswd, dbPort => $squidGuardConf->dbPort);
if ($squidGuardDB->didErrorOccur)
{
  myDie(error => $squidGuardDB->errorMessage);
}

# now lookup the server they specified.
my $sth = $squidGuardDB->read(sql => "SELECT id, passwd, version, bl_groups FROM servers_tb WHERE cid = ? AND server = ?", plug => [ $companyId, $serverName ]);
if ($squidGuardDB->didErrorOccur)
{
  myDie(error => $squidGuardDB->errorMessage);
}
my @info = $sth->fetchrow_array;
$sth->finish;
if (!defined $info[0])
{
  myDie(error => "serverName = '$serverName' does not exist for companyCode = '$companyCode'!");
}
$serverId = $info[0];
$realPass = $info[1];
$dbVersion = $info[2];
$serverBLGroups = $info[3];

# now validate the password given is correct.
if ($realPass ne $serverPass)
{
  myDie(error => "Passwords do not match!");
}

# now check and see if the firewall's version is old.
if ($dbVersion > $serverVersion)
{
  # build the updated tarball and send it to them.
  my $filesToTar = "allowed VERSION";

  # make sure the /tmp/squidGuard directory exists
  system("/bin/mkdir -p -m 0700 /tmp/squidGuard") == 0 or myDie(error => "Could not create /tmp/squidGuard.  $!");

  # generate a unique file name in the /tmp directory under /tmp/squidGuard
  my $fileName = tmpnam();

  # replace /tmp with /tmp/squidGuard
  $fileName =~ s/^(\/tmp)/$1\/squidGuard/;

  # now create the directory
  system("/bin/mkdir -p $fileName") == 0 or myDie(error => "Could not create $fileName.  $!");

  # now create the allowed directory
  system("/bin/mkdir -p $fileName/allowed") == 0 or myDie(error => "Could not create $fileName/allowed.  $!");

  # gather the allowed domains and urls entries and output them.
  foreach my $file (qw(domains urls))
  {
    (my $entry = $file) =~ s/s$//;
    $sth = $squidGuardDB->read(sql => "SELECT $entry FROM allowed_" . $file . "_tb WHERE sid = ? ORDER BY $entry", plug => [ $serverId ]);
    if ($squidGuardDB->didErrorOccur)
    {
      myDie(error => $squidGuardDB->errorMessage);
    }

    my $fHandle = createFile("$fileName/allowed/$file");
    while (my @info = $sth->fetchrow_array)
    {
      print $fHandle $info[0] . "\n";
    }
    $sth->finish;
    close($fHandle);
  }

  # create the VERSION file
  my $fHandle = createFile("$fileName/VERSION");
  writeFile($fHandle, $dbVersion);

  # see if we need to create the blacklists directory
  if (length $serverBLGroups > 0)
  {
    $filesToTar .= " blacklists";
    system("/bin/mkdir -p $fileName/blacklists") == 0 or myDie(error => "Could not create $fileName/blacklists.  $!");
    my @groups = split /\|/, $serverBLGroups;

    foreach my $group (@groups)
    {
      # create the group directory.
      system("/bin/mkdir -p $fileName/blacklists/$group") == 0 or myDie(error => "Could not create $fileName/blacklists/$group.  $!");

      # gather the allowed domains and urls entries and output them.
      foreach my $file (qw(domains urls))
      {
        (my $entry = $file) =~ s/s$//;
        $sth = $squidGuardDB->read(sql => "SELECT $entry FROM bl_" . $file . "_tb WHERE sid = ? AND bl_group = ? ORDER BY $entry", plug => [ $serverId, $group ]);
        if ($squidGuardDB->didErrorOccur)
        {
          myDie(error => $squidGuardDB->errorMessage);
        }

        my $fHandle = createFile("$fileName/blacklists/$group/$file.local");
        while (my @info = $sth->fetchrow_array)
        {
          print $fHandle $info[0] . "\n";
        }
        $sth->finish;
        close($fHandle);
      }
    }
  }

  # build the tar ball directly into the body of the document.
  my $result = `/bin/tar --ignore-failed-read -cz -C $fileName $filesToTar`;
  $doc->print($result);

  # remove the temporary directory.
  system("/bin/rm -rf $fileName") == 0 or myDie(error => "Could not remove $fileName.  $!");
}
else
{
  $doc->setContentType("text/plain");  # switch back to a text document.
  $doc->setFocus("body");
  $doc->print("No Change\n");
}

$doc->display;  # display the generated document.

exit 1;

# myDie - Takes error
sub myDie
{
  my %args = ( error => "", @_ );
  my $message = $args{error};
  my $doc = undef;

  $doc = HTMLObject::Base->new;
  $doc->setContentType("text/plain");

  $doc->setFocus("body");
  $doc->print("Error\n");
  $doc->print($message . "\n");

  $doc->display();

  exit 1;
}

# helper functions

sub openFile
{
  my $fname = shift;

  if (length $fname == 0)
  {
    myDie(error => "Error:  You must specify the filename to open!");
  }
  if (! -e $fname)
  {
    die "Error:  file = $fname does not exist!\n";
  }
  open(F, "+< $fname") or myDie(error => "Error:  Could not open file $fname!  $!");
  return *F;
}

sub createFile
{
  my $fname = shift;

  if (length $fname == 0)
  {
    myDie(error => "Error:  You must specify the filename to open!");
  }
  open(F, "> $fname") or myDie(error => "Error:  Could not create file $fname!  $!");
  return *F;
}

sub readFile
{
  my $fh = shift;

  my $source = "";

  seek ($fh, 0, 0) or myDie(error => "Error:  Could not seek to start of file: $!");

  while (<$fh>)
  {
    $source .= $_;  # read in the file
  }

  return $source;
}

sub writeFile
{
  my $fh = shift;
  my $source = shift;

  seek ($fh, 0, 0) or myDie(error => "Error:  Could not seek to start of file: $!");
  print $fh $source or myDie(error => "Error: Could not print to file: $!");
  truncate($fh, tell($fh)) or myDie(error => "Error: Could not truncate file: $!");
  close ($fh) or myDie(error => "Error: Could not close file: $!");
}
