# Config.pm - The Object Class that provides a Config Data Object
# Created by James A. Pattie, 11/07/2000.

# Copyright (c) 2000-2002 Xperience, Inc. http://www.pcxperience.com/
#
#This program is free software; you can redistribute it and/or
#modify it under the terms of the GNU General Public License
#as published by the Free Software Foundation; either version 2
#of the License, or (at your option) any later version.
#
#This program is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#GNU General Public License for more details.
#
#You should have received a copy of the GNU General Public License
#along with this program; if not, write to the Free Software
#Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

package Portal::squidGuard::Data::Config;
use strict;
use vars qw($AUTOLOAD $VERSION @ISA @EXPORT);

require Exporter;

@ISA = qw(Exporter AutoLoader);
@EXPORT = qw();

$VERSION = '0.01';

my $configVersion = "0.1";

# new
# instantiates an instance of a Config Object
sub new
{
  my $that = shift;
  my $class = ref($that) || $that;
  my $self = bless {}, $class;
  my %args = (  @_ );
  my $errStr = "Portal::squidGuard::Data::Config->new()  - Error!<br>\n";

  # source in the config file from /etc/pcx_portal/squidGuard/Config.pm and suck the settings hash into ourselves.
  my $cfgFile = "/etc/pcx_portal/squidGuard/Config.pm";
  my $return = "";
  unless ($return = do $cfgFile)
  {
    die $errStr . "Couldn't parse '$cfgFile': $@<br>\n" if $@;
    die $errStr . "Couldn't do '$cfgFile': $!<br>\n" unless defined $return;
    die $errStr . "Couldn't run '$cfgFile'" unless $return;
  }
  my %settings = ();
  %settings = %PortalsquidGuardSettings::settings;  # <--- Change App to your App Name! ----------

  foreach my $name (keys %settings)
  {
    $self->{$name} = $settings{$name};
  }

  # do validation
  $self->{error} = !$self->isValid;
  if ($self->{error})
  {
    $self->{errorString} = $errStr . $self->{errorString};
  }

  if ($self->{configVersion} ne $configVersion)
  {
    die $errStr . "Config File '$cfgFile' is at version '$self->{configVersion}' not at the current version of '$configVersion'!<br>\n";
  }

  return $self;
}

# isValid - Returns 0 or 1 to indicate if the object is valid.
sub isValid
{
  my $self = shift;
  my $error = 0;
  my $errorString = "";
  my $errStr = "Portal::squidGuard::Data::Config->isValid()  - Error!<br>\n";

  # do validation code here.

  if (length $errorString > 0)
  {
    $error = 1;
    $self->{error} = $error;
    $self->{errorString} = $errStr;
    $self->{errorString} .= $errorString;
  }

  return !$error;
}

sub DESTROY
{
  my $self = shift;
}

sub AUTOLOAD  # Read Only
{
  my $self = shift;
  my $type = ref($self) || die "$self is not an object";
  my $name = $AUTOLOAD;
  $name =~ s/.*://;	# strip fully-qualified portion
  unless (exists $self->{$name})
  {
    die "Can't access `$name' field in object of class $type";
  }
  return $self->{$name};
}

# setError
# requires: errorString
# returns: nothing
sub setError
{
  my $self = shift;
  my %args = ( errorString => "", @_ );
  my $errorString = $args{errorString};

  $self->{errorString} = $errorString;
  $self->{error} = 1;
}

# didErrorOccur
# returns the value of error.
sub didErrorOccur
{
  my $self = shift;

  return $self->{error};
}

# errorMessage
# returns the value of errorString
sub errorMessage
{
  my $self = shift;

  return $self->{errorString};
}

1;
__END__

=head1 NAME

Config - Object used to build a Config Data Object Class.

=head1 SYNOPSIS

  use Portal::squidGuard::Data::Config;
  my $obj = undef;
  eval { $obj = Portal::squidGuard::Data::Config->new; };
  if ($@)
  {
    die "Error instantiating Config Object!  $@\n";
  }
  if ($obj->didErrorOccur())
  {
    die $obj->errorMessage();
  }

=head1 DESCRIPTION

Config is a Config class.

=head1 Exported FUNCTIONS

  scalar new(void)
    Creates a new instance of the Portal::squidGuard::Data::Config
    object.

  bool isValid(void)
    Returns 0 or 1 to indicate if the object is valid.  The error will
    be available via errorMessage().
    
  void setError(errorString)
    Sets error = 1 and errorString = errorString.

  scalar didErrorOccur(void)
    Returns the value of error.

  scalar errorMessage(void)
    Returns the value of errorString.

=head1 Exported VARIABLES

  dbName                - set to appName_db if this app needs its own
                          database.
  dbPerCompany          - set to true if this app needs its own database
                          for each company.
  appType               - administration, help, bugs, regular, tools,
                          news
  appAdminType          - normal or system
  appDescription        - describe (briefly) what the app does.  This
                          will be used in the App purchasing and
                          assigning Apps.
  appIconName           - name the icon this app will display on the
                          desktop.
  height                - default height of the app.
  width                 - default width of the app.
  autorun               - do you want this app to automatically startup
                          when the user logs in?
  dbType                - Database backend to use. Pg or mysql
  dbPort                - The Port to connect to for this database
                          server. 3306 for mysql, 5432 for Pg. Username
                          and password will come from the portals
                          settings.
  appName               - This is the name of the Applications perl
                          module.  Case sensitive.
  isAppFree             - if false, then app is a for fee app.
  assignToAllCompanies  - if false, don't assign to any companies.
  assignToAllUsers      - if true, then assign to all users, else check
                          the admin and sysadmin flags.
  assignToAdminUsers    - if true, then assign to all admin and sysadmin
                          users, else check sysadmin flag.
  assignToSysAdminUsers - if true, then assign to only sysadmin users.
  appCost               - The default cost of the app.  0.00 indicates
                          free.
  unit                  - 0 = per month, 1 = per user.
  numLicenses           - The number of licenses to assign to a company
                          if assignToAllCompanies is true.  0 means
                          unlimited.
  wapAware              - Is this app WAP aware?

  NOTE:  All data fields are accessible by specifying the object
         and pointing to the data member to be modified on the
         left-hand side of the assignment.
         Ex.  $obj->variable($newValue); or $value = $obj->variable;

=head1 AUTHOR

PC & Web Xperience, Inc. (mailto:admin@pcxperience.com)

=head1 SEE ALSO

perl(1), Portal(3), Portal::squidGuard(3)

=cut
