# Language.pm - The Object Class that provides a Language Object
# Created by James A. Pattie, 11/07/2000.

# Copyright (c) 2000-2002 Xperience, Inc. http://www.pcxperience.com/
#
#This program is free software; you can redistribute it and/or
#modify it under the terms of the GNU General Public License
#as published by the Free Software Foundation; either version 2
#of the License, or (at your option) any later version.
#
#This program is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#GNU General Public License for more details.
#
#You should have received a copy of the GNU General Public License
#along with this program; if not, write to the Free Software
#Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

package Portal::squidGuard::Language;
use strict;
use HTMLObject::Base;  # needed for the language stuff.
use vars qw($AUTOLOAD $VERSION @ISA @EXPORT);

require Exporter;

@ISA = qw(Exporter AutoLoader);
@EXPORT = qw();

$VERSION = '0.03';

# new
# instantiates an instance of a Language class
sub new
{
  my $that = shift;
  my $class = ref($that) || $that;
  my $self = bless {}, $class;
  my %args = ( lang => 'en', @_ );
  my $errStr = "Portal::squidGuard::Language->new() - Error!<br>\n";

  $self->{lang} = $args{lang};

  my $doc = HTMLObject::Base->new;
  my $module_name = $doc->lookupLanguageName(code => $self->{lang});
  if (length $module_name == 0) # it wasn't found, default to English.
  {
    $module_name = "English";
    $self->{lang} = "en";
  }
  if (!-e "Language/$module_name". ".pm")
  {
    $module_name = "English";
    $self->{lang} = "en";
  }
  $module_name = "Portal::squidGuard::Language::" . $module_name;

  # bring it into existance.
  eval "use $module_name;";
  if ($@)
  {
    $self->setError(errorString => $errStr . "Eval of $module_name failed!<br>\nError = '$@'.<br>\n");

    return $self;
  }
  $self->{phraseObj} = $module_name->new;  # Instantiate the Language Phrase object.  It will contain the charEncoding to use.
  if ($self->{phraseObj}->didErrorOccur)
  {
    $self->setError(errorString => $errStr . $self->{phraseObj}->errorMessage);

    return $self;
  }

  # do validation
  $self->{error} = !$self->isValid;
  if ($self->{error})
  {
    $self->{errorString} = $errStr . $self->{errorString};
  }

  return $self;
}

# isValid - Returns 0 or 1 to indicate if the object is valid.
sub isValid
{
  my $self = shift;
  my $error = 0;
  my $errorString = "";
  my $errStr = "Portal::squidGuard::Language->isValid() - Error!<br>\n";

  # do validation code here.

  if (length $errorString > 0)
  {
    $error = 1;
    $self->{error} = $error;
    $self->{errorString} = $errStr;
    $self->{errorString} .= $errorString;
  }

  return !$error;
}

sub DESTROY
{
  my $self = shift;
}

sub AUTOLOAD   # Read only, setting does not work.
{
  my $self = shift;
  my $type = ref($self) || die "$self is not an object";
  my $name = $AUTOLOAD;
  $name =~ s/.*://;	# strip fully-qualified portion
  unless (exists $self->{$name})
  {
    die "Can't access `$name' field in object of class $type";
  }
  return $self->{$name};
}

# setError
# requires: errorString
# returns: nothing
sub setError
{
  my $self = shift;
  my %args = ( errorString => "", @_ );
  my $errorString = $args{errorString};
  
  $self->{errorString} = $errorString;
  $self->{error} = 1;
}

# didErrorOccur
# returns the value of error.
sub didErrorOccur
{
  my $self = shift;

  return $self->{error};
}

# errorMessage
# returns the value of errorString
sub errorMessage
{
  my $self = shift;
  
  return $self->{errorString};
}

1;
__END__

=head1 NAME

Language - Object used to build a Language Object Class.

=head1 SYNOPSIS

  use Portal::squidGuard::Language;
  my $obj = Portal::squidGuard::Language->new(lang => $lang);
  if ($obj->didErrorOccur())
  {
    die $obj->errorMessage();
  }

=head1 DESCRIPTION

Language is a Language class.

=head1 Exported FUNCTIONS

  scalar new(lang)
    Creates a new instance of the Portal::squidGuard::Language
    object.  Internally sources the specified language if able and
    creates an internal data object that holds the language hash to
    work with for the Portal.

  bool isValid(void)
    Returns 0 or 1 to indicate if the object is valid.  The error will be
    available via errorMessage().
    
  void setError(errorString)
    Sets error = 1 and errorString = errorString.

  scalar didErrorOccur(void)
    Returns the value of error.

  scalar errorMessage(void)
    Returns the value of errorString.

=head1 Exported VARIABLES

  LanguageObject phraseObj - The Language that has phrases and encoding.

  NOTE:  All data fields are accessible by specifying the object
         and pointing to the data member to be modified on the
         left-hand side of the assignment.
         Ex.  $obj->variable($newValue); or $value = $obj->variable;

=head1 AUTHOR

PC & Web Xperience, Inc. (mailto:admin@pcxperience.com)

=head1 SEE ALSO

perl(1), Portal(3), Portal::squidGuard(3)

=cut
