# Template.pm - The Object Class that provides a Template Language Object
# Created by James A. Pattie, 11/07/2000.

# Copyright (c) 2000-2002 Xperience, Inc. http://www.pcxperience.com/
#
#This program is free software; you can redistribute it and/or
#modify it under the terms of the GNU General Public License
#as published by the Free Software Foundation; either version 2
#of the License, or (at your option) any later version.
#
#This program is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#GNU General Public License for more details.
#
#You should have received a copy of the GNU General Public License
#along with this program; if not, write to the Free Software
#Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

package Portal::squidGuard::Language::Template;
use strict;
use vars qw($AUTOLOAD $VERSION @ISA @EXPORT);

require Exporter;

@ISA = qw(Exporter AutoLoader);
@EXPORT = qw();

$VERSION = '0.01';

# The Phrases used by the App.  - Add these to the Template and convert in each language as created.

my %phrases = {};
$phrases{stateNeeded} = "The State of the Application must be specified!";
$phrases{commandNeeded} = "The Command to execute must be specified!";
$phrases{sessionCookieNeeded} = "The Session Cookie must be defined!";
$phrases{sessionNotCreated} = "The Session does not exist but Session Cookie is defined!";
$phrases{commandNotKnown} = "Command = '%s' is not known!";
$phrases{stateNotKnown} = "State = '%s' is not known!";
$phrases{stateEvalFailed} = "Eval of State = '%s' failed!<br>\nError = '%s'.";
$phrases{appStateCommand} = "App = '%s', State = '%s', Command = '%s'.";
$phrases{close} = "Close App";
$phrases{help} = "Help";
$phrases{appVersion} = "App Version = <font color=\"%s\">%s</font>";
$phrases{refresh} = "Refresh";
$phrases{title} = "squidGuard %s - %s";
$phrases{mainMenu} = "Main Menu";
$phrases{closeMessage} = "Press 'OK' if you really want to Close this Application,\\nelse press 'Cancel'.";
$phrases{domainAndUrl} = "Domains And Urls";
$phrases{error} = "Error!";
$phrases{serverSettings} = "Server Settings";
$phrases{settings} = "Settings";
$phrases{rules} = "Rules";
$phrases{selectall} = "Select All";
$phrases{unselectall} = "UnSelect All";
$phrases{edit} = "Edit";
$phrases{delete} = "Delete";
$phrases{update} = "Update";
$phrases{domains} = "Domains";
$phrases{urls} = "Urls";
$phrases{export} = "Export";
$phrases{selectServer} = "Select Server";
$phrases{server} = "Server";
$phrases{create} = "Create";
$phrases{select} = "Select";
$phrases{selectServerToWorkWith} = "Select a server to work with.";
$phrases{noServersCreated} = "You do not have any servers configured!";
$phrases{gotoSettings} = "Goto <b>Settings</b> to create one.";
$phrases{allowed} = "Allowed";
$phrases{blackLists} = "Black Lists";
$phrases{noGroupsSelected} = "No groups selected. &nbsp;Goto the <b>Settings</b> to select.";
$phrases{serverInvalid} = "Server = '%s' is invalid!";
$phrases{selectServerEdit} = "You must select a server to edit!";
$phrases{selectServerDelete} = "You must select a server to delete!";
$phrases{confirmServerDelete} = "Do you want to delete this server?\\nOK to Delete, Cancel to Keep.";
$phrases{serverRequired} = "You must specify the server to work with!";
$phrases{serverInvalidOrNotFound} = "Server ID = '%s' is invalid or does not belong to you!";
$phrases{password} = "Password";
$phrases{exportMessage} = "Press 'OK' if you really want to Increment the Version of your squidGuard DB files,\\nelse press 'Cancel'.";
$phrases{menu} = "Menu";

# new
# instantiates an instance of the Template Language Phrases.
sub new
{
  my $that = shift;
  my $class = ref($that) || $that;
  my $self = bless {}, $class;
  my %args = (  @_ );
  my $errStr = "Portal::squidGuard::Language::Template->new() - Error!<br>\n";

  $self->{phrases} = { %phrases };
  $self->{encoding} = "iso-8859-1";

  # do validation
  $self->{error} = !$self->isValid;
  if ($self->{error})
  {
    $self->{errorString} = $errStr . $self->{errorString};
  }

  return $self;
}

# isValid - Returns 0 or 1 to indicate if the object is valid.
sub isValid
{
  my $self = shift;
  my $error = 0;
  my $errorString = "";
  my $errStr = "Portal::squidGuard::Language::Template->isValid() - Error!<br>\n";

  # do validation code here.

  if (length $errorString > 0)
  {
    $error = 1;
    $self->{error} = $error;
    $self->{errorString} = $errStr;
    $self->{errorString} .= $errorString;
  }

  return !$error;
}

sub DESTROY
{
  my $self = shift;
}

sub AUTOLOAD  # act as though all calls are just queries, setting of values will not work.
{
  my $self = shift;
  my $type = ref($self) || die "$self is not an object";
  my $name = $AUTOLOAD;
  $name =~ s/.*://;	# strip fully-qualified portion
  unless (exists $self->{$name})
  {
    die "Can't access `$name' field in object of class $type";
  }
  return $self->{$name};
}

# setError
# requires: errorString
# returns: nothing
sub setError
{
  my $self = shift;
  my %args = ( errorString => "", @_ );
  my $errorString = $args{errorString};
  
  $self->{errorString} = $errorString;
  $self->{error} = 1;
}

# didErrorOccur
# returns the value of error.
sub didErrorOccur
{
  my $self = shift;

  return $self->{error};
}

# errorMessage
# returns the value of errorString
sub errorMessage
{
  my $self = shift;
  
  return $self->{errorString};
}

1;
__END__

=head1 NAME

Template - Object used to build a Template Object Class.

=head1 SYNOPSIS

  use Portal::squidGuard::Language::Template;
  my $obj = Portal::squidGuard::Language::Template->new;
  if ($obj->didErrorOccur())
  {
    die $obj->errorMessage();
  }

=head1 DESCRIPTION

Template is a Template Language class.

=head1 Exported FUNCTIONS

  scalar new(void)
    Creates a new instance of the Portal::squidGuard::Language::Template
    object.

  bool isValid(void)
    Returns 0 or 1 to indicate if the object is valid.  The error will be
    available via errorMessage().
    
  void setError(errorString)
    Sets error = 1 and errorString = errorString.

  scalar didErrorOccur(void)
    Returns the value of error.

  scalar errorMessage(void)
    Returns the value of errorString.

=head1 Exported VARIABLES

  hash   phrases  - converted phrases
  scalar encoding - content type encoding for document.

  NOTE:  All data fields are accessible by specifying the object
         and pointing to the data member to be modified on the
         left-hand side of the assignment.
         Ex.  $obj->variable($newValue); or $value = $obj->variable;

=head1 AUTHOR

PC & Web Xperience, Inc. (mailto:admin@pcxperience.com)

=head1 SEE ALSO

perl(1), Portal(3), Portal::squidGuard(3)

=cut
