# Rules.pm - The Object Class that provides a Rules Object
# Created by James A. Pattie, 10/31/2001.

# Copyright (c) 2000-2002 Xperience, Inc. http://www.pcxperience.com/
#
#This program is free software; you can redistribute it and/or
#modify it under the terms of the GNU General Public License
#as published by the Free Software Foundation; either version 2
#of the License, or (at your option) any later version.
#
#This program is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#GNU General Public License for more details.
#
#You should have received a copy of the GNU General Public License
#along with this program; if not, write to the Free Software
#Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

package Portal::squidGuard::Rules;
use strict;
use Portal::Data::Variables;
use Portal::squidGuard::Data::Config;
use vars qw($AUTOLOAD $VERSION @ISA @EXPORT);

require Exporter;

@ISA = qw(Exporter AutoLoader);
@EXPORT = qw();

$VERSION = '0.01';

# global variables.
# modify to represent the commands this state provides.
my %commands = ( display => "Show Database Entries", deleterule => "Delete specified Rules", editrule => "Mark the selected rules as being edited or insert new rules.", export => "Generate the .local files.", );

# new
# instantiates an instance of a Rules
# takes: langObj, input, cookieObj, dbHandle, logObj, sessionObj, userObj, companyObj, browserType, browserVersion, browserMaker, authObj
sub new
{
  my $that = shift;
  my $class = ref($that) || $that;
  my $self = bless {}, $class;
  my %args = ( langObj => undef, input => undef, cookieObj => undef, dbHandle => undef, logObj => undef, sessionObj => undef, userObj => undef, companyObj => undef,
               applicationObj => undef, appObj => undef, portalSession => undef, browserType => "HTML", browserVersion => "4.0", browserMaker => "", authObj => undef, @_ );
  my $errStr = "Portal::squidGuard::Rules->new() - Error!<br>\n";

  my $configObj = undef;
  eval { $configObj = Portal::squidGuard::Data::Config->new; };
  if ($@)
  {
    $self->setError(errorString => $errStr . "eval of Portal::squidGuard::Data::Config->new failed!<br>\nError = '$@'<br>\n");
    return $self;
  }
  $self->{configObj} = $configObj;
  $self->{langObj} = $args{langObj};
  $self->{input} = $args{input};
  $self->{cookieObj} = $args{cookieObj};
  $self->{dbHandle} = $args{dbHandle};
  $self->{logObj} = $args{logObj};
  $self->{sessionObj} = $args{sessionObj};
  $self->{userObj} = $args{userObj};
  $self->{companyObj} = $args{companyObj};
  $self->{applicationObj} = $args{applicationObj};
  $self->{appObj} = $args{appObj};
  $self->{portalSession} = $args{portalSession};
  $self->{browserType} = $args{browserType};
  $self->{browserVersion} = $args{browserVersion};
  $self->{browserMaker} = $args{browserMaker};
  $self->{authObj} = $args{authObj};

  # do validation
  $self->{error} = !$self->isValid;
  if ($self->{error})
  {
    $self->{errorString} = $errStr . $self->{errorString};
  }

  return $self;
}

# isValid - Returns 0 or 1 to indicate if the object is valid.
sub isValid
{
  my $self = shift;
  my $error = 0;
  my $errorString = "";
  my $errStr = "Portal::squidGuard::Rules->isValid() - Error!<br>\n";

  # do validation code here.
  if (not defined $self->{langObj})
  {
    $errorString .= "langObj must be defined!<br>\n";
  }
  elsif (!$self->{langObj}->isValid)
  {
    $errorString .= $self->{langObj}->errorMessage;
  }
  if (not defined $self->{applicationObj})
  {
    $errorString .= "applicationObj must be defined!<br>\n";
  }
  elsif (!$self->{applicationObj}->isValid)
  {
    $errorString .= $self->{applicationObj}->errorMessage;
  }
  if (not defined $self->{appObj})
  {
    $errorString .= "appObj must be defined!<br>\n";
  }
  elsif (!$self->{appObj}->isValid)
  {
    $errorString .= $self->{appObj}->errorMessage;
  }
  if (not defined $self->{portalSession})
  {
    $errorString .= "portalSession must be defined!<br>\n";
  }
  elsif (!$self->{portalSession}->isValid)
  {
    $errorString .= $self->{portalSession}->errorMessage;
  }
  if (not defined $self->{userObj})
  {
    $errorString .= "userObj must be defined!<br>\n";
  }
  elsif (!$self->{userObj}->isValid)
  {
    $errorString .= $self->{userObj}->errorMessage;
  }
  if (not defined $self->{companyObj})
  {
    $errorString .= "companyObj must be defined!<br>\n";
  }
  elsif (!$self->{companyObj}->isValid)
  {
    $errorString .= $self->{companyObj}->errorMessage;
  }
  if (not defined $self->{authObj})
  {
    $errorString .= "authObj must be defined!<br>\n";
  }
  elsif (!$self->{authObj}->isValid)
  {
    $errorString .= $self->{authObj}->errorMessage;
  }
  if (not defined $self->{logObj})
  {
    $errorString .= "logObj must be defined!<br>\n";
  }
  elsif (!$self->{logObj}->isValid)
  {
    $errorString .= $self->{logObj}->errorMessage;
  }
  if (not defined $self->{cookieObj})
  {
    $errorString .= "cookieObj must be defined!<br>\n";
  }
  if (not defined $self->{dbHandle})
  {
    $errorString .= "dbHandle must be defined!<br>\n";
  }
  if (not defined $self->{sessionObj})
  {
    $errorString .= "sessionObj must be defined!<br>\n";
  }
  if ($self->{browserType} !~ /^(HTML|HDML|WML)$/)
  {
    $errorString .= "browserType = '$self->{browserType}' is invalid!<br>\n";
  }
  if (length $self->{browserVersion} == 0)
  {
    $errorString .= "browserType = '$self->{browserType}' is invalid!<br>\n";
  }

  if (length $errorString > 0)
  {
    $error = 1;
    $self->{error} = $error;
    $self->{errorString} = $errStr;
    $self->{errorString} .= $errorString;
  }

  return !$error;
}

sub DESTROY
{
  my $self = shift;
}

sub AUTOLOAD
{
  my $self = shift;
  my $type = ref($self) || die "$self is not an object";
  my $name = $AUTOLOAD;
  $name =~ s/.*://;	# strip fully-qualified portion
  unless (exists $self->{$name})
  {
    die "Can't access `$name' field in object of class $type";
  }
  if (@_)
  {
    return $self->{$name} = shift;
  }
  else
  {
    return $self->{$name};
  }
}

# setError
# requires: errorString
# returns: nothing
sub setError
{
  my $self = shift;
  my %args = ( errorString => "", @_ );
  my $errorString = $args{errorString};

  $self->{errorString} = $errorString;
  $self->{error} = 1;
}

# didErrorOccur
# returns the value of error.
sub didErrorOccur
{
  my $self = shift;

  return $self->{error};
}

# errorMessage
# returns the value of errorString
sub errorMessage
{
  my $self = shift;

  return $self->{errorString};
}

# run
# takes: command
# returns: The HTMLObject document to display or undef if an error.
sub run
{
  my $self = shift;
  my %args = ( command => "", @_ );
  my $command = $args{command};
  my $doc;
  my $phrases = $self->{langObj}->{phraseObj}->{phrases};
  my $errorPhrase = $phrases->{error};
  my $errStr = "Portal::squidGuard::Rules->run()  - $errorPhrase<br>\n";

  if (!exists $commands{$command})
  {
    $self->setError(errorString => $errStr . sprintf($phrases->{commandNotKnown}, $command));
    return undef;
  }

  $doc = HTMLObject::Normal->new;
  if ($command eq "display")
  {
    $doc = $self->displayRules(doc => $doc, input => $self->{input});
    if ($self->didErrorOccur)
    {
      $self->setError(errorString => $errStr . $self->errorMessage);
      return undef;
    }
  }
  elsif ($command eq "deleterule")
  {
    # make sure we got type, group, file and entries
    my $type = $self->{input}->{type};
    my $group = $self->{input}->{group};
    my $file = $self->{input}->{file};
    my $entries = $self->{input}->{entries};
    my $serverID = $self->{input}->{serverID};

    if ($type !~ /^(allowed|bl)$/)
    {
      $self->setError(errorString => $errStr . "type = '$type' is invalid!<br>\n");
      return undef;
    }
    if ($type eq "bl")
    {
      if ($group !~ /^(ads|aggressive|audio-video|drugs|gambling|hacking|mail|porn|proxy|violence|warez)$/)
      {
        $self->setError(errorString => $errStr . "group = '$group' is invalid!<br>\n");
        return undef;
      }
    }
    if ($file !~ /^(domains|urls)$/)
    {
      $self->setError(errorString => $errStr . "file = '$file' is invalid!<br>\n");
      return undef;
    }
    if ($serverID !~ /^(\d+)$/)
    {
      $self->setError(errorString => $errStr . "serverID = '$serverID' is invalid!<br>\n");
      return undef;
    }

    my @entries = split /\0/, $entries;
    (my $field = $file) =~ s/s$//;  # remove the trailing s.

    foreach my $entry (@entries)
    {
      next if (length $entry == 0);
      my $extraOrder = "";
      my $sql = "DELETE FROM ";
      if ($type eq "allowed")
      {
        $sql .= "allowed_" . $file . "_tb";
      }
      elsif ($type eq "bl")
      {
        $sql .= "bl_" . $file . "_tb";
        $extraOrder = " AND bl_group = '$group'";
      }
      $sql .= " WHERE sid = ? AND $field = ?$extraOrder";
      $self->{dbHandle}->write(sql => $sql, plug => [ $serverID, $entry ]);
      if ($self->{dbHandle}->didErrorOccur)
      {
        $self->setError(errorString => $errStr . $self->{dbHandle}->errorMessage);
        return undef;
      }
    }

    $self->{dbHandle}->commit;
    if ($self->{dbHandle}->didErrorOccur)
    {
      $self->setError(errorString => $errStr . $self->{dbHandle}->errorMessage);
      return undef;
    }

    # now redisplay the main screen for this type and group
    $doc = $self->displayRules(doc => $doc, input => $self->{input});
    if ($self->didErrorOccur)
    {
      $self->setError(errorString => $errStr . $self->errorMessage);
      return undef;
    }
  }
  elsif ($command eq "editrule")
  {
    # make sure we got type, group, file, hiddenEntries and entries
    my $type = $self->{input}->{type};
    my $group = $self->{input}->{group};
    my $file = $self->{input}->{file};
    my $hiddenEntries = $self->{input}->{hiddenEntries};
    my $entries = $self->{input}->{entries};
    my $serverID = $self->{input}->{serverID};

    if ($type !~ /^(allowed|bl)$/)
    {
      $self->setError(errorString => $errStr . "type = '$type' is invalid!<br>\n");
      return undef;
    }
    if ($type eq "bl")
    {
      if ($group !~ /^(ads|aggressive|audio-video|drugs|gambling|hacking|mail|porn|proxy|violence|warez)$/)
      {
        $self->setError(errorString => $errStr . "group = '$group' is invalid!<br>\n");
        return undef;
      }
    }
    if ($file !~ /^(domains|urls)$/)
    {
      $self->setError(errorString => $errStr . "file = '$file' is invalid!<br>\n");
      return undef;
    }
    if ($serverID !~ /^(\d+)$/)
    {
      $self->setError(errorString => $errStr . "serverID = '$serverID' is invalid!<br>\n");
      return undef;
    }

    my @removeThese = split /\|/, $hiddenEntries;
    my @entries = split /\r\n/, $entries;
    (my $field = $file) =~ s/s$//;  # remove the trailing s.

    # see if we have to remove any entries first.
    if (scalar @removeThese > 0)
    {
      foreach my $entry (@removeThese)
      {
        next if (length $entry == 0);
        my $extraOrder = "";
        my $sql = "DELETE FROM ";
        if ($type eq "allowed")
        {
          $sql .= "allowed_" . $file . "_tb";
        }
        elsif ($type eq "bl")
        {
          $sql .= "bl_" . $file . "_tb";
          $extraOrder = " AND bl_group = '$group'";
        }
        $sql .= " WHERE sid = ? AND $field = ?$extraOrder";
        $self->{dbHandle}->write(sql => $sql, plug => [ $serverID, $entry ]);
        if ($self->{dbHandle}->didErrorOccur)
        {
          $self->setError(errorString => $errStr . $self->{dbHandle}->errorMessage);
          return undef;
        }
      }
    }

    # now insert the new entries.
    foreach my $entry (@entries)
    {
      next if (length $entry == 0);
      my $sql = "INSERT INTO ";
      my $result = 0;
      if ($type eq "allowed")
      {
        $sql .= "allowed_" . $file . "_tb (sid, $field) VALUES (?, ?)";
        $result = $self->{dbHandle}->write(sql => $sql, plug => [ $serverID, $entry ]);
      }
      elsif ($type eq "bl")
      {
        $sql .= "bl_" . $file . "_tb (sid, bl_group, $field) VALUES (?, ?, ?)";
        $result = $self->{dbHandle}->write(sql => $sql, plug => [ $serverID, $group, $entry ]);
      }
      if ($self->{dbHandle}->didErrorOccur)
      {
        $self->setError(errorString => $errStr . $self->{dbHandle}->errorMessage);
        return undef;
      }
      if ($result != 1)
      {
        $self->setError(errorString => $errStr . "Insert of '$sql', sid = '$serverID', group = '$group', entry = '$entry' had '$result' entries inserted!<br>\n");
        return undef;
      }
    }

    $self->{dbHandle}->commit;
    if ($self->{dbHandle}->didErrorOccur)
    {
      $self->setError(errorString => $errStr . $self->{dbHandle}->errorMessage);
      return undef;
    }

    # now redisplay the main screen for this type and group
    $doc = $self->displayRules(doc => $doc, input => $self->{input});
    if ($self->didErrorOccur)
    {
      $self->setError(errorString => $errStr . $self->errorMessage);
      return undef;
    }
  }
  elsif ($command eq "export")
  {
    # we increment the version number by one and just display the menu without the export link and a status message of success.
    my $serverID = $self->{input}->{serverID};

    if ($serverID !~ /^(\d+)$/)
    {
      $self->setError(errorString => $errStr . "serverID = '$serverID' is invalid!<br>\n");
      return undef;
    }

    # get the current version from the database.
    my $sth = $self->{dbHandle}->read(sql => "SELECT server, version FROM servers_tb WHERE id = ? AND cid = ? FOR UPDATE", plug => [ $serverID, $self->{companyObj}->{id} ]);
    if ($self->{dbHandle}->didErrorOccur)
    {
      $self->setError(errorString => $errStr . $self->{dbHandle}->errorMessage);
      return undef;
    }
    my @info = $sth->fetchrow_array;
    $sth->finish;
    if (!defined $info[0])
    {
      $self->setError(errorString => $errStr . "No data returned when getting current version for server with ID = '$serverID'!<br>\n");
      return undef;
    }
    my $serverName = $info[0];
    my $version = $info[1];

    # increment the version by 1
    $version++;

    # now update the database.
    $self->{dbHandle}->write(sql => "UPDATE servers_tb SET version = ? WHERE id = ? AND cid = ?", plug => [ $version, $serverID, $self->{companyObj}->{id} ]);
    if ($self->{dbHandle}->didErrorOccur)
    {
      $self->setError(errorString => $errStr . $self->{dbHandle}->errorMessage);
      return undef;
    }

    $self->{dbHandle}->commit;
    if ($self->{dbHandle}->didErrorOccur)
    {
      $self->setError(errorString => $errStr . $self->{dbHandle}->errorMessage);
      return undef;
    }

    # now display the main screen with the status
    $doc = $self->displayMenu(doc => $doc, command => "export");
    if ($self->didErrorOccur)
    {
      $self->setError(errorString => $errStr . $self->errorMessage);
      return undef;
    }
    $doc->setFocus("body");
    $doc->print("<br><h1>Version incremented to '$version' for server = '$serverName'.</h1>\n");
  }

  return $doc;
}

sub displayRules
{
  my $self = shift;
  my %args = ( doc => undef, error => "", status => "", @_ );
  my $doc = $args{doc};
  my $error = $args{error};
  my $status = $args{status};
  my $phrases = $self->{langObj}->{phraseObj}->{phrases};
  my $errorPhrase = $phrases->{error};
  my $errStr = "Portal::squidGuard::User->displayUserPreferences()  - $errorPhrase<br>\n";

  my $bgColor = $self->{configObj}->{bgColor};
  my $fgColor = $self->{configObj}->{fgColor};
  my $menufgColor = $self->{configObj}->{menufgColor};
  my $menubgColor = $self->{configObj}->{menubgColor};
  my $cellfgColor = $self->{configObj}->{cellfgColor};
  my $cellbgColor = $self->{configObj}->{cellbgColor};
  my $cellTitlefgColor = $self->{configObj}->{cellTitlefgColor};
  my $cellTitlebgColor = $self->{configObj}->{cellTitlebgColor};
  my $tablebgColor = $self->{configObj}->{tablebgColor};

  my $selectAllPhrase = $phrases->{selectall};
  my $unselectAllPhrase = $phrases->{unselectall};
  my $editPhrase = $phrases->{edit};
  my $deletePhrase = $phrases->{delete};
  my $updatePhrase = $phrases->{update};
  my $domainsPhrase = $phrases->{domains};
  my $urlsPhrase = $phrases->{urls};

  $doc->setFocus("javascript");
  $doc->print(<<"END_OF_CODE");

function selectAllEntries(item)
{
  for (var i=0; i < item.options.length; i++)
  {
    item.options[i].selected = true;
  }
}

function unselectAllEntries(item)
{
  for (var i=0; i < item.options.length; i++)
  {
    item.options[i].selected = false;
  }
}

function editEntries(entries, editField, hiddenField)
{
  if (entries.selectedIndex == -1)
  {
    alert("You must select entries to Edit!");
    return;
  }
  if (editField.value.length > 0 || hiddenField.value.length > 0)
  {
    if (!window.confirm("You already have something being edited!\\nPress OK to Replace,\\nPress Cancel to keep the contents."))
    {
      return;
    }
    else
    {
      hiddenField.value = "";
      editField.value = "";
    }
  }
  // Find out which entries are selected and put them in the hidden field in the edit form
  for (var i=0; i < entries.options.length; i++)
  {
    if (entries.options[i].selected)
    {
      if (hiddenField.value.length > 0)
      {
        hiddenField.value += "|";
        editField.value += "\\n";
      }
      hiddenField.value += entries.options[i].value;
      editField.value += entries.options[i].value;
    }
  }
}

function validateDelete(entries)
{
  if (entries.selectedIndex == -1)
  {
    alert("You must select entries to Delete!");
    return false;
  }
  return window.confirm("Do you want to Delete these entries?\\nPress Cancel to abort or OK to continue...");
}

function validateUpdate(entries)
{
  if (entries.value.length == 0)
  {
    alert("You must specify entries to Update/Create!");
    return false;
  }
  return true;
}

END_OF_CODE

  my $configObj = Portal::Data::Config->new;
  my $url = ($self->{sessionObj}->{store}->{companyAppObj}->{port} == 80 ? "http" : "https") . "://" . $self->{sessionObj}->{store}->{companyAppObj}->{server} . $configObj->{siteDir} . "/cgi-bin/index.cgi";

  $doc = $self->displayMenu(doc => $doc, command => "display");
  if ($self->didErrorOccur)
  {
    $self->setError(errorString => $errStr . $self->errorMessage);
    return $doc;
  }

  $doc->print("<font color=\"red\">Error</font>: $error<br>\n") if (length $error > 0);
  $doc->print("Status: $status<br>\n") if (length $status > 0);

  # make sure we are given the data we need (type and group).
  my $type = $self->{input}->{type};
  my $group = $self->{input}->{group};
  my $serverID = $self->{input}->{serverID};
  my $serverName = "";
  my $header = "";
  if ($type !~ /^(allowed|bl)$/)
  {
    $self->setError(errorString => $errStr . "type = '$type' is invalid!<br>\n");
    return undef;
  }
  if ($type eq "bl")
  {
    if ($group !~ /^(ads|aggressive|audio-video|drugs|gambling|hacking|mail|porn|proxy|violence|warez)$/)
    {
      $self->setError(errorString => $errStr . "group = '$group' is invalid!<br>\n");
      return undef;
    }
    $header = "Black Lists - $group";
  }
  else
  {
    $header = "Allowed";
  }
  if ($serverID !~ /^(\d+)$/)
  {
    $self->setError(errorString => $errStr . "serverID = '$serverID' is invalid!<br>\n");
    return undef;
  }
  my $sth = $self->{dbHandle}->read(sql => "SELECT server FROM servers_tb WHERE id = ?", plug => [ $serverID ]);
  if ($self->{dbHandle}->didErrorOccur)
  {
    $self->setError(errorString => $errStr . $self->{dbHandle}->errorMessage);
    return undef;
  }
  my @info = $sth->fetchrow_array;
  if (!defined $info[0])
  {
    $self->setError(errorString => $errStr . "No server exists with id = '$serverID'!<br>\n");
    return undef;
  }
  $serverName = $info[0];
  $sth->finish;

  # start the table display code which will help align both domains and urls.
  $doc->print(<<"END_OF_CODE");
<br>
$serverName: $header
<table border="0" cellpadding="2" cellspacing="0" bgColor="$tablebgColor">
END_OF_CODE

  # now go into a loop and handle the domains and then the urls output.
  foreach my $file (qw(domains urls))
  {
    # display the header
    $doc->print(<<"END_OF_CODE");
  <tr bgColor="$cellTitlebgColor">
    <th align="center"><font color="$cellTitlefgColor">$file</font></th>
  </tr>
END_OF_CODE

    # determine if there are any entries in this file
    (my $field = $file) =~ s/s$//;  # remove the trailing s.
    my $extraOrder = "";
    my $sql = "SELECT COUNT($field) FROM ";
    if ($type eq "allowed")
    {
      $sql .= "allowed_" . $file . "_tb";
    }
    elsif ($type eq "bl")
    {
      $sql .= "bl_" . $file . "_tb";
      $extraOrder = " AND bl_group = '$group'";
    }
    $sql .= " WHERE sid = ?$extraOrder";
    my $sth = $self->{dbHandle}->read(sql => $sql, plug => [ $serverID ]);
    if ($self->{dbHandle}->didErrorOccur)
    {
      $self->setError(errorString => $errStr . $self->{dbHandle}->errorMessage);
      return undef;
    }
    my @info = $sth->fetchrow_array;
    $sth->finish;

    # if so, start the form and display the entries in a multi-select box along with edit and delete buttons
    if ($info[0] > 0)
    {
      my $sql = "SELECT $field FROM ";
      if ($type eq "allowed")
      {
        $sql .= "allowed_" . $file . "_tb";
      }
      elsif ($type eq "bl")
      {
        $sql .= "bl_" . $file . "_tb";
        $extraOrder = " AND bl_group = '$group'";
      }
      $sql .= " WHERE sid = ?$extraOrder ORDER BY $field";
      my $sth = $self->{dbHandle}->read(sql => $sql, plug => [ $serverID ]);
      if ($self->{dbHandle}->didErrorOccur)
      {
        $self->setError(errorString => $errStr . $self->{dbHandle}->errorMessage);
        return undef;
      }

      my $formName = $file . "_entries_form";
      my $editFormName = $file . "_edit_form";
      my $selectName = "entries";
      my $selectAllName = "selectAll$file";
      my $unselectAllName = "unselectAll$file";
      my $editName = "edit$file";
      my $deleteName = "delete$file";

      $doc->print(<<"END_OF_CODE");
  <tr>
    <td align="center">
      <form name="$formName" method="post" action="$url">
        <input type="hidden" name="app" value="squidGuard">
        <input type="hidden" name="state" value="Rules">
        <input type="hidden" name="command" value="deleterule">
        <input type="hidden" name="type" value="$type">
        <input type="hidden" name="group" value="$group">
        <input type="hidden" name="file" value="$file">
        <input type="hidden" name="serverID" value="$serverID">
        <table border="0" cellpadding="2" cellspacing="0" width="100%">
          <tr>
            <td align="center">
              <select name="$selectName" multiple size="10">
END_OF_CODE

      while (my @info = $sth->fetchrow_array)
      {
        $doc->print("                <option value=\"$info[0]\">$info[0]\n");
      }
      $sth->finish;

      $doc->print(<<"END_OF_CODE");
              </select>
            </td>
            <td align="center">
              <input type="button" name="$selectAllName" value="$selectAllPhrase" onClick="selectAllEntries(document.$formName.$selectName)"><br>
              <input type="button" name="$unselectAllName" value="$unselectAllPhrase" onClick="unselectAllEntries(document.$formName.$selectName)"><br>
              <br>
              <br>
              <input type="button" name="$editName" value="$editPhrase" onClick="editEntries(document.$formName.$selectName, document.$editFormName.entries, document.$editFormName.hiddenEntries)"><br>
              <input type="button" name="$deleteName" value="$deletePhrase" onClick="if(validateDelete(document.$formName.$selectName)) document.$formName.submit();">
            </td>
          </tr>
        </table>
      </form>
    </td>
  </tr>
END_OF_CODE
    }
    # else, output that there are no entries
    else
    {
      $doc->print(<<"END_OF_CODE");
  <tr>
    <td align="center"><b>NO</b> entries created.</td>
  </tr>
END_OF_CODE
    }

    # create the edit/new entries form along with the hidden form item that the edit button will fill in but will default to be empty
    my $formName = $file . "_edit_form";

    if ($file eq "domains")
    {
      $doc->print(<<"END_OF_CODE");
  <tr>
    <td align="left"><font size="-1"><b>domains</b> entries are in the format fully qualified host name or just a domain name or an IP Address.<br>
    Ex.  <i>(each entry must be on its own line)</i><br>
    www.pcxperience.com<br>
    w3.org<br>
    192.168.100.1
    </font>
    </td>
  </tr>
END_OF_CODE
    }
    elsif ($file eq "urls")
    {
      $doc->print(<<"END_OF_CODE");
  <tr>
    <td align="left"><font size="-1"><b>urls</b> entries are in the format of domain name or IP Address / <i>something</i>.<br>
    Ex.  <i>(each entry must be on its own line)</i><br>
    pcxperience.com/news/<br>
    192.168.100.1/news/index.html
    </font>
    </td>
  </tr>
END_OF_CODE
    }

    $doc->print(<<"END_OF_CODE");
  <tr>
    <td align="center">
      <form name="$formName" method="post" action="$url">
        <input type="hidden" name="app" value="squidGuard">
        <input type="hidden" name="state" value="Rules">
        <input type="hidden" name="command" value="editrule">
        <input type="hidden" name="type" value="$type">
        <input type="hidden" name="group" value="$group">
        <input type="hidden" name="file" value="$file">
        <input type="hidden" name="hiddenEntries" value="">
        <input type="hidden" name="serverID" value="$serverID">
        <textarea name="entries" rows="8" cols="40" wrap="off"></textarea><br>
        <input type="button" name="update" value="Update" onClick="if(validateUpdate(document.$formName.entries)) document.$formName.submit();">
      </form>
    </td>
  </tr>
END_OF_CODE
  }

  # close the table
  $doc->print("</table>\n</center>\n");

  return $doc
}

sub displayMenu
{
  my $self = shift;
  my %args = ( doc => undef, command => "", @_ );
  my $doc = $args{doc};
  my $command = $args{command};
  my $phrases = $self->{langObj}->{phraseObj}->{phrases};
  my $errorPhrase = $phrases->{error};
  my $errStr = "Portal::squidGuard::User->displayMenu()  - $errorPhrase<br>\n";

  my $bgColor = $self->{configObj}->{bgColor};
  my $fgColor = $self->{configObj}->{fgColor};
  my $menufgColor = $self->{configObj}->{menufgColor};
  my $menubgColor = $self->{configObj}->{menubgColor};

  my $close = $phrases->{close};
  my $refresh = $phrases->{refresh};
  my $help = $phrases->{help};
  my $mainMenu = $phrases->{mainMenu};
  my $rulesPhrase = $phrases->{rules};
  my $title = sprintf($phrases->{title}, $Portal::squidGuard::VERSION, $rulesPhrase);

  $doc->setTitle($title);

  $doc->setFocus("javascript");
  $doc->print(<<"END_OF_CODE");

function showHelp(appName, url, height, width)
{
  var w = window.open(url, appName, "height="+height+",width="+width+",resizable,scrollbars,status");
}

END_OF_CODE
  $doc->setLinkDecorations();  # turn off underlines on the links.

  my $serverName = "";
  my $sth = $self->{dbHandle}->read(sql => "SELECT server FROM servers_tb where id = ?", plug => [ $self->{input}->{serverID} ]);
  if ($self->{dbHandle}->didErrorOccur)
  {
    $self->setError(errorString => $errStr . $self->{dbHandle}->errorMessage);
    return undef;
  }
  my @info = $sth->fetchrow_array;
  $sth->finish;
  $serverName = $info[0];

  my $configObj = Portal::Data::Config->new;
  my $url = ($self->{sessionObj}->{store}->{companyAppObj}->{port} == 80 ? "http" : "https") . "://" . $self->{sessionObj}->{store}->{companyAppObj}->{server} . $configObj->{siteDir} . "/cgi-bin/index.cgi";
  my $mainMenuUrl = $url . "?app=squidGuard&state=Main&command=display&server=" . $serverName;
  my $refreshUrl = $url . "?app=squidGuard&state=Rules&command=display&type=". $self->{input}->{type} . "&group=" . $self->{input}->{group} . "&serverID=" . $self->{input}->{serverID};
  my $helpUrl = $url . "?app=Portal&state=Help&command=display&helpApp=squidGuard&content=Normal,DB+Rules";

  $doc->setFocus("body");
  $doc->print(<<"END_OF_CODE");
<center>
<table border="0" width="100%" cellspacing="0" cellpadding="2" bgcolor="$menubgColor">
  <tr>
    <td align="left" width="100%"><a href="$mainMenuUrl" onMouseOver="window.status='$mainMenu'; return true;" title="$mainMenu"><font color="$menufgColor">$mainMenu</font></a> &nbsp;| &nbsp;
    <a href="$helpUrl" onMouseOver="window.status='$help'; return true;" title="$help" onClick="showHelp('help_squidGuard', '$helpUrl', 480, 640); return false;"><font color="$menufgColor">$help</font></a>
END_OF_CODE
  if ($command ne "export")
  {
    $doc->print(<<"END_OF_CODE");
    &nbsp;| &nbsp;<a href="$refreshUrl" onMouseOver="window.status='$refresh'; return true;" title="$refresh"><font color="$menufgColor">$refresh</font></a></td>
END_OF_CODE
  }
  $doc->print(<<"END_OF_CODE");
  </tr>
</table>
END_OF_CODE

  return $doc;
}

1;
__END__

=head1 NAME

Rules - Object used to build a Rules Object Class.

=head1 SYNOPSIS

  use Portal::squidGuard::Rules;
  my $obj = Portal::squidGuard::Rules->new;
  if ($obj->didErrorOccur())
  {
    die $obj->errorMessage();
  }
  $obj->run(command => $command);

=head1 DESCRIPTION

Rules is a Rules class.

=head1 Exported FUNCTIONS

  scalar new(langObj, input, cookieObj, dbHandle, logObj, sessionObj,
             userObj, companyObj, applicationObj, appObj, portalSession,
             browserType, browserVersion, browserMaker, authObj)
    Creates a new instance of the Portal::squidGuard::Rules
    object.

  bool isValid(void)
    Returns 0 or 1 to indicate if the object is valid.  The error will 
    be available via errorMessage().
    
  void setError(errorString)
    Sets error = 1 and errorString = errorString.

  scalar didErrorOccur(void)
    Returns the value of error.

  scalar errorMessage(void)
    Returns the value of errorString.
    
  HTMLObject run(command)
    Runs the specified command and returns the resulting HTMLObject.


  NOTE:  All data fields are accessible by specifying the object
         and pointing to the data member to be modified on the
         left-hand side of the assignment.
         Ex.  $obj->variable($newValue); or $value = $obj->variable;

=head1 AUTHOR

PC & Web Xperience, Inc. (mailto:admin@pcxperience.com)

=head1 SEE ALSO

perl(1), Portal(3), Portal::squidGuard(3)

=cut
