# Server.pm - The Object Class that provides a Server Object
# Created by James A. Pattie, 10/30/2001.

# Copyright (c) 2000-2002 Xperience, Inc. http://www.pcxperience.com/
#
#This program is free software; you can redistribute it and/or
#modify it under the terms of the GNU General Public License
#as published by the Free Software Foundation; either version 2
#of the License, or (at your option) any later version.
#
#This program is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#GNU General Public License for more details.
#
#You should have received a copy of the GNU General Public License
#along with this program; if not, write to the Free Software
#Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

package Portal::squidGuard::Server;
use strict;
use Portal::Data::Variables;
use Portal::squidGuard::Data::Config;
use vars qw($AUTOLOAD $VERSION @ISA @EXPORT);

require Exporter;

@ISA = qw(Exporter AutoLoader);
@EXPORT = qw();

$VERSION = '0.01';

# global variables.
# modify to represent the commands this state provides.
my %commands = ( display => "Display Servers", edit => "Server Preferences", create => "Create Server", delete => "Delete Server", );

# new
# instantiates an instance of a Server
# takes: langObj, input, cookieObj, dbHandle, logObj, sessionObj, userObj, companyObj, browserType, browserVersion, browserMaker, authObj
sub new
{
  my $that = shift;
  my $class = ref($that) || $that;
  my $self = bless {}, $class;
  my %args = ( langObj => undef, input => undef, cookieObj => undef, dbHandle => undef, logObj => undef, sessionObj => undef, userObj => undef, companyObj => undef,
               applicationObj => undef, appObj => undef, portalSession => undef, browserType => "HTML", browserVersion => "4.0", browserMaker => "", authObj => undef, @_ );
  my $errStr = "Portal::squidGuard::Server->new() - Error!<br>\n";

  $self->{portalVariables} = Portal::Data::Variables->new;
  my $configObj = undef;
  eval { $configObj = Portal::squidGuard::Data::Config->new; };
  if ($@)
  {
    $self->setError(errorString => $errStr . "eval of Portal::squidGuard::Data::Config->new failed!<br>\nError = '$@'<br>\n");
    return $self;
  }
  $self->{configObj} = $configObj;
  $self->{langObj} = $args{langObj};
  $self->{input} = $args{input};
  $self->{cookieObj} = $args{cookieObj};
  $self->{dbHandle} = $args{dbHandle};
  $self->{logObj} = $args{logObj};
  $self->{sessionObj} = $args{sessionObj};
  $self->{userObj} = $args{userObj};
  $self->{companyObj} = $args{companyObj};
  $self->{applicationObj} = $args{applicationObj};
  $self->{appObj} = $args{appObj};
  $self->{portalSession} = $args{portalSession};
  $self->{browserType} = $args{browserType};
  $self->{browserVersion} = $args{browserVersion};
  $self->{browserMaker} = $args{browserMaker};
  $self->{authObj} = $args{authObj};

  # do validation
  $self->{error} = !$self->isValid;
  if ($self->{error})
  {
    $self->{errorString} = $errStr . $self->{errorString};
  }

  return $self;
}

# isValid - Returns 0 or 1 to indicate if the object is valid.
sub isValid
{
  my $self = shift;
  my $error = 0;
  my $errorString = "";
  my $errStr = "Portal::squidGuard::Server->isValid() - Error!<br>\n";

  # do validation code here.
  if (not defined $self->{langObj})
  {
    $errorString .= "langObj must be defined!<br>\n";
  }
  elsif (!$self->{langObj}->isValid)
  {
    $errorString .= $self->{langObj}->errorMessage;
  }
  if (not defined $self->{applicationObj})
  {
    $errorString .= "applicationObj must be defined!<br>\n";
  }
  elsif (!$self->{applicationObj}->isValid)
  {
    $errorString .= $self->{applicationObj}->errorMessage;
  }
  if (not defined $self->{appObj})
  {
    $errorString .= "appObj must be defined!<br>\n";
  }
  elsif (!$self->{appObj}->isValid)
  {
    $errorString .= $self->{appObj}->errorMessage;
  }
  if (not defined $self->{portalSession})
  {
    $errorString .= "portalSession must be defined!<br>\n";
  }
  elsif (!$self->{portalSession}->isValid)
  {
    $errorString .= $self->{portalSession}->errorMessage;
  }
  if (not defined $self->{userObj})
  {
    $errorString .= "userObj must be defined!<br>\n";
  }
  elsif (!$self->{userObj}->isValid)
  {
    $errorString .= $self->{userObj}->errorMessage;
  }
  if (not defined $self->{companyObj})
  {
    $errorString .= "companyObj must be defined!<br>\n";
  }
  elsif (!$self->{companyObj}->isValid)
  {
    $errorString .= $self->{companyObj}->errorMessage;
  }
  if (not defined $self->{authObj})
  {
    $errorString .= "authObj must be defined!<br>\n";
  }
  elsif (!$self->{authObj}->isValid)
  {
    $errorString .= $self->{authObj}->errorMessage;
  }
  if (not defined $self->{logObj})
  {
    $errorString .= "logObj must be defined!<br>\n";
  }
  elsif (!$self->{logObj}->isValid)
  {
    $errorString .= $self->{logObj}->errorMessage;
  }
  if (not defined $self->{cookieObj})
  {
    $errorString .= "cookieObj must be defined!<br>\n";
  }
  if (not defined $self->{dbHandle})
  {
    $errorString .= "dbHandle must be defined!<br>\n";
  }
  if (not defined $self->{sessionObj})
  {
    $errorString .= "sessionObj must be defined!<br>\n";
  }
  if ($self->{browserType} !~ /^(HTML|HDML|WML)$/)
  {
    $errorString .= "browserType = '$self->{browserType}' is invalid!<br>\n";
  }
  if (length $self->{browserVersion} == 0)
  {
    $errorString .= "browserType = '$self->{browserType}' is invalid!<br>\n";
  }

  if (length $errorString > 0)
  {
    $error = 1;
    $self->{error} = $error;
    $self->{errorString} = $errStr;
    $self->{errorString} .= $errorString;
  }

  return !$error;
}

sub DESTROY
{
  my $self = shift;
}

sub AUTOLOAD
{
  my $self = shift;
  my $type = ref($self) || die "$self is not an object";
  my $name = $AUTOLOAD;
  $name =~ s/.*://;	# strip fully-qualified portion
  unless (exists $self->{$name})
  {
    die "Can't access `$name' field in object of class $type";
  }
  if (@_)
  {
    return $self->{$name} = shift;
  }
  else
  {
    return $self->{$name};
  }
}

# setError
# requires: errorString
# returns: nothing
sub setError
{
  my $self = shift;
  my %args = ( errorString => "", @_ );
  my $errorString = $args{errorString};

  $self->{errorString} = $errorString;
  $self->{error} = 1;
}

# didErrorOccur
# returns the value of error.
sub didErrorOccur
{
  my $self = shift;

  return $self->{error};
}

# errorMessage
# returns the value of errorString
sub errorMessage
{
  my $self = shift;

  return $self->{errorString};
}

# run
# takes: command
# returns: The HTMLObject document to display or undef if an error.
sub run
{
  my $self = shift;
  my %args = ( command => "", @_ );
  my $command = $args{command};
  my $doc;
  my $phrases = $self->{langObj}->{phraseObj}->{phrases};
  my $errorPhrase = $phrases->{error};
  my $errStr = "Portal::squidGuard::Server->run()  - $errorPhrase<br>\n";

  if (!exists $commands{$command})
  {
    $self->setError(errorString => $errStr . sprintf($phrases->{commandNotKnown}, $command));
    return undef;
  }

  $doc = HTMLObject::Normal->new;
  if ($command eq "display")
  {
    $doc = $self->displayServers(doc => $doc);
    if ($self->didErrorOccur)
    {
      $self->setError(errorString => $errStr . $self->errorMessage);
      return undef;
    }
  }
  elsif ($command eq "edit")
  {
    if (not exists $self->{input}->{update}) # display the Servers Preferences
    {
      $doc = $self->displayServerPreferences(doc => $doc);
      if ($self->didErrorOccur)
      {
        $self->setError(errorString => $errStr . $self->errorMessage);
        return undef;
      }
    }
    else  # try to update the Servers Preferences
    {
      # make sure I have the correct stuff passed in.
      my $serverName = $self->{input}->{serverName};
      my $oldServerName = $self->{input}->{oldServerName};
      my $serverPass = $self->{input}->{serverPass};
      my $blGroups = $self->{input}->{blGroups};

      my $errorString = "";
      if (length $oldServerName == 0)
      {
        $errorString .= "You must specify the old server name!<br>\n";
      }
      if (length $serverName == 0)
      {
        $errorString .= "You must specify the server name!<br>\n";
      }
      if (length $serverPass < 6)
      {
        $errorString .= "You must specify a password that is 6 or more characters in length!<br>\n";
      }
      if (length $errorString > 0)
      {
        $doc = $self->displayServerPreferences(doc => $doc, error => $errorString);
        if ($self->didErrorOccur)
        {
          $self->setError(errorString => $errStr . $self->errorMessage);
          return undef;
        }
      }
      else  # update the database.
      {
        # make sure the server exists (lookup oldServerName)
        my $sth = $self->{dbHandle}->read(sql => "SELECT id FROM servers_tb WHERE cid = ? AND server = ?", plug => [ $self->{companyObj}->{id}, $oldServerName ]);
        if ($self->{dbHandle}->didErrorOccur)
        {
          $self->setError(errorString => $errStr . $self->{dbHandle}->errorMessage);
          return undef;
        }
        my @info = $sth->fetchrow_array;
        if (not defined $info[0])
        {
          $doc = $self->displayServerPreferences(doc => $doc, error => "Server = '$oldServerName' doesn't exist!<br>\n");
          if ($self->didErrorOccur)
          {
            $self->setError(errorString => $errStr . $self->errorMessage);
            return undef;
          }
          return $doc;
        }
        my $serverId = $info[0];
        $sth->finish;

        (my $tmpblGroups = $blGroups) =~ s/\0/\|/g;  # fixup the blGroups value to go into the database.
        $self->{dbHandle}->write(sql => "UPDATE servers_tb SET server = ?, bl_groups = ?, passwd = ? WHERE id = ?",
                                 plug => [ $serverName, $tmpblGroups, $serverPass, $serverId ]);
        if ($self->{dbHandle}->didErrorOccur)
        {
          $self->setError(errorString => $errStr . $self->{dbHandle}->errorMessage);
          return undef;
        }
        $self->{dbHandle}->commit;  # save the changes.
        if ($self->{dbHandle}->didErrorOccur)
        {
          $self->setError(errorString => $errStr . $self->{dbHandle}->errorMessage);
          return undef;
        }

        $doc = $self->displayServerPreferences(doc => $doc, status => "Successfully updated preferences!");
        if ($self->didErrorOccur)
        {
          $self->setError(errorString => $errStr . $self->errorMessage);
          return undef;
        }
      }
    }
  }
  elsif ($command eq "create")
  {
    if (not exists $self->{input}->{update}) # display the Create Servers Preferences
    {
      $doc = $self->displayCreateServer(doc => $doc);
      if ($self->didErrorOccur)
      {
        $self->setError(errorString => $errStr . $self->errorMessage);
        return undef;
      }
    }
    else  # try to update the Servers Preferences
    {
      # make sure I have the correct stuff passed in.
      my $serverName = $self->{input}->{serverName};
      my $serverPass = $self->{input}->{serverPass};
      my $blGroups = $self->{input}->{blGroups};

      my $errorString = "";
      if (length $serverName == 0)
      {
        $errorString .= "You must specify the server name!<br>\n";
      }
      if (length $serverPass < 6)
      {
        $errorString .= "You must specify a password that is 6 or more characters in length!<br>\n";
      }
      if (length $errorString > 0)
      {
        $doc = $self->displayCreateServer(doc => $doc, error => $errorString);
        if ($self->didErrorOccur)
        {
          $self->setError(errorString => $errStr . $self->errorMessage);
          return undef;
        }
        return $doc;
      }
      else  # update the database.
      {
        # make sure the server doesn't exist
        my $sth = $self->{dbHandle}->read(sql => "SELECT id FROM servers_tb WHERE cid = ? AND server = ?", plug => [ $self->{companyObj}->{id}, $serverName ]);
        if ($self->{dbHandle}->didErrorOccur)
        {
          $self->setError(errorString => $errStr . $self->{dbHandle}->errorMessage);
          return undef;
        }
        my @info = $sth->fetchrow_array;
        if (defined $info[0])
        {
          $doc = $self->displayCreateServer(doc => $doc, error => "Server = '$serverName' already exists!<br>\n");
          if ($self->didErrorOccur)
          {
            $self->setError(errorString => $errStr . $self->errorMessage);
            return undef;
          }
          return $doc;
        }
        $sth->finish;

        (my $tmpblGroups = $blGroups) =~ s/\0/\|/g;  # fixup the blGroups value to go into the database.
        $self->{dbHandle}->write(sql => "INSERT INTO servers_tb (cid, server, bl_groups, passwd) VALUES (?, ?, ?, ?)",
                                 plug => [ $self->{companyObj}->{id}, $serverName, $tmpblGroups, $serverPass ]);
        if ($self->{dbHandle}->didErrorOccur)
        {
          $self->setError(errorString => $errStr . $self->{dbHandle}->errorMessage);
          return undef;
        }
        $self->{dbHandle}->commit;  # save the changes.
        if ($self->{dbHandle}->didErrorOccur)
        {
          $self->setError(errorString => $errStr . $self->{dbHandle}->errorMessage);
          return undef;
        }

        # findout what the server id is we just inserted.
        $sth = $self->{dbHandle}->read(sql => "SELECT id FROM servers_tb WHERE cid = ? AND server = ?", plug => [ $self->{companyObj}->{id}, $serverName ]);
        if ($self->{dbHandle}->didErrorOccur)
        {
          $self->setError(errorString => $errStr . $self->{dbHandle}->errorMessage);
          return undef;
        }
        @info = $sth->fetchrow_array;
        if (not defined $info[0])
        {
          $doc = $self->displayCreateServer(doc => $doc, error => "Server = '$serverName' doesn't exist and we just created it!<br>\n");
          if ($self->didErrorOccur)
          {
            $self->setError(errorString => $errStr . $self->errorMessage);
            return undef;
          }
          return $doc;
        }
        my $serverId = $info[0];
        $sth->finish;

        $self->{input}->{serverID} = $serverId;
        delete $self->{input}->{serverName};
        $doc = $self->displayServerPreferences(doc => $doc, status => "Successfully updated preferences!");
        if ($self->didErrorOccur)
        {
          $self->setError(errorString => $errStr . $self->errorMessage);
          return undef;
        }
      }
    }
  }
  elsif ($command eq "delete")
  {
    my $serverID = $self->{input}->{serverID};
    if (length $serverID == 0)
    {
      $self->setError(errorString => $errStr . "You must specify a server to delete!<br>\n");
      return undef;
    }
    my $sth = $self->{dbHandle}->read(sql => "SELECT server FROM servers_tb WHERE cid = ? AND id = ?", plug => [ $self->{companyObj}->{id}, $serverID ]);
    if ($self->{dbHandle}->didErrorOccur)
    {
      $self->setError(errorString => $errStr . $self->{dbHandle}->errorMessage);
      return undef;
    }
    my @info = $sth->fetchrow_array;
    if (not defined $info[0])
    {
      $doc = $self->displayServerPreferences(doc => $doc, error => "Server with ID = '$serverID' doesn't exist!<br>\n");
      if ($self->didErrorOccur)
      {
        $self->setError(errorString => $errStr . $self->errorMessage);
        return undef;
      }
    }
    my $serverName = $info[0];
    $sth->finish;

    $self->{dbHandle}->write(sql => "DELETE FROM servers_tb WHERE id = ?",
                            plug => [ $serverID ]);
    if ($self->{dbHandle}->didErrorOccur)
    {
      $self->setError(errorString => $errStr . $self->{dbHandle}->errorMessage);
      return undef;
    }

    # now delete all allowed and black list entries for this server
    $self->{dbHandle}->write(sql => "DELETE FROM allowed_domains_tb WHERE sid = ?", plug => [ $serverID ]);
    if ($self->{dbHandle}->didErrorOccur)
    {
      $self->setError(errorString => $errStr . $self->{dbHandle}->errorMessage);
      return undef;
    }

    $self->{dbHandle}->write(sql => "DELETE FROM allowed_urls_tb WHERE sid = ?", plug => [ $serverID ]);
    if ($self->{dbHandle}->didErrorOccur)
    {
      $self->setError(errorString => $errStr . $self->{dbHandle}->errorMessage);
      return undef;
    }

    $self->{dbHandle}->write(sql => "DELETE FROM bl_domains_tb WHERE sid = ?", plug => [ $serverID ]);
    if ($self->{dbHandle}->didErrorOccur)
    {
      $self->setError(errorString => $errStr . $self->{dbHandle}->errorMessage);
      return undef;
    }

    $self->{dbHandle}->write(sql => "DELETE FROM bl_urls_tb WHERE sid = ?", plug => [ $serverID ]);
    if ($self->{dbHandle}->didErrorOccur)
    {
      $self->setError(errorString => $errStr . $self->{dbHandle}->errorMessage);
      return undef;
    }

    $self->{dbHandle}->commit;  # save the changes.
    if ($self->{dbHandle}->didErrorOccur)
    {
      $self->setError(errorString => $errStr . $self->{dbHandle}->errorMessage);
      return undef;
    }

    $doc = $self->displayServers(doc => $doc, status => "Successfully deleted <b>$serverName</b>.");
    if ($self->didErrorOccur)
    {
      $self->setError(errorString => $errStr . $self->errorMessage);
      return undef;
    }
  }

  return $doc;
}

sub displayServers
{
  my $self = shift;
  my %args = ( doc => undef, error => "", status => "", @_ );
  my $doc = $args{doc};
  my $error = $args{error};
  my $status = $args{status};
  my $phrases = $self->{langObj}->{phraseObj}->{phrases};
  my $errorPhrase = $phrases->{error};
  my $errStr = "Portal::squidGuard::Server->displayServerPreferences()  - $errorPhrase<br>\n";

  my $bgColor = $self->{configObj}->{bgColor};
  my $fgColor = $self->{configObj}->{fgColor};
  my $menufgColor = $self->{configObj}->{menufgColor};
  my $menubgColor = $self->{configObj}->{menubgColor};
  my $cellfgColor = $self->{configObj}->{cellfgColor};
  my $cellbgColor = $self->{configObj}->{cellbgColor};
  my $cellTitlefgColor = $self->{configObj}->{cellTitlefgColor};
  my $cellTitlebgColor = $self->{configObj}->{cellTitlebgColor};
  my $tablebgColor = $self->{configObj}->{tablebgColor};

  my $close = $phrases->{close};
  my $refresh = $phrases->{refresh};
  my $help = $phrases->{help};
  my $mainMenu = $phrases->{mainMenu};
  my $closeMessage = $phrases->{closeMessage};
  my $selectServerPhrase = $phrases->{selectServer};
  my $title = sprintf($phrases->{title}, $Portal::squidGuard::VERSION, $selectServerPhrase);
  my $editPhrase = $phrases->{edit};
  my $deletePhrase = $phrases->{delete};
  my $createPhrase = $phrases->{create};
  my $serverPhrase = $phrases->{server};
  my $selectServerEditPhrase = $phrases->{selectServerEdit};
  my $selectServerDeletePhrase = $phrases->{selectServerDelete};
  my $confirmServerDeletePhrase = $phrases->{confirmServerDelete};

  $doc->setTitle($title);

  $doc->setFocus("javascript");
  $doc->print(<<"END_OF_CODE");

function showHelp(appName, url, height, width)
{
  var w = window.open(url, appName, "height="+height+",width="+width+",resizable,scrollbars,status");
}

function validateEdit()
{
  var d = document.prefs.serverID;

  if (d.selectedIndex == -1)
  {
    alert("$selectServerEditPhrase");
    return false;
  }
  return true;
}

function validateDelete()
{
  var d = document.prefs.serverID;

  if (d.selectedIndex == -1)
  {
    alert("$selectServerDeletePhrase");
    return false;
  }
  return window.confirm("$confirmServerDeletePhrase");
}

END_OF_CODE
  $doc->setLinkDecorations();  # turn off underlines on the links.

  my $configObj = Portal::Data::Config->new;
  my $url = ($self->{sessionObj}->{store}->{companyAppObj}->{port} == 80 ? "http" : "https") . "://" . $self->{sessionObj}->{store}->{companyAppObj}->{server} . $configObj->{siteDir} . "/cgi-bin/index.cgi";
  my $mainMenuUrl = $url . "?app=squidGuard&state=Main&command=display";
  my $refreshUrl = $url . "?app=squidGuard&state=Server&command=display";
  my $helpUrl = $url . "?app=Portal&state=Help&command=display&helpApp=squidGuard&content=Normal,Server+Settings";

  $doc->setFocus("body");
  $doc->print(<<"END_OF_CODE");
<center>
<table border="0" width="100%" cellspacing="0" cellpadding="2" bgcolor="$menubgColor">
  <tr>
    <td align="left" width="100%"><a href="$mainMenuUrl" onMouseOver="window.status='$mainMenu'; return true;" title="$mainMenu"><font color="$menufgColor">$mainMenu</font></a> &nbsp;| &nbsp;
    <a href="$helpUrl" onMouseOver="window.status='$help'; return true;" title="$help" onClick="showHelp('help_squidGuard', '$helpUrl', 480, 640); return false;"><font color="$menufgColor">$help</font></a> &nbsp;| &nbsp;
    <a href="$refreshUrl" onMouseOver="window.status='$refresh'; return true;" title="$refresh"><font color="$menufgColor">$refresh</font></a></td>
  </tr>
</table>
END_OF_CODE

  $doc->print("<font color=\"red\">Error</font>: $error<br>\n") if (length $error > 0);
  $doc->print("Status: $status<br>\n") if (length $status > 0);

  my $sth = $self->{dbHandle}->read(sql => "SELECT server, id FROM servers_tb WHERE cid = ?", plug => [ $self->{companyObj}->{id} ]);
  if ($self->{dbHandle}->didErrorOccur)
  {
    $self->setError(errorString => $errStr . $self->{dbHandle}->errorMessage);
    return $doc;
  }
  my @servers = ();
  my %servers = ();
  while (my @info = $sth->fetchrow_array)
  {
    $servers[++$#servers] = $info[0];
    $servers{$info[0]} = $info[1];
  }
  $sth->finish;

  $doc->print(<<"END_OF_CODE");
<br>
<form name="prefs" method="post" action="$url">
  <input type="hidden" name="app" value="squidGuard">
  <input type="hidden" name="state" value="Server">
  <input type="hidden" name="command" value="edit">
END_OF_CODE
  if (scalar @servers > 0)
  {
    $doc->print(<<"END_OF_CODE");
  <table border="0" cellpadding="2" cellspacing="0" bgColor="$tablebgColor">
    <tr>
      <td align="center">
        Server <select name="serverID">
END_OF_CODE
  foreach my $server (@servers)
  {
    $doc->print("            <option value=\"$servers{$server}\">$server\n");
  }
  $doc->print(<<"END_OF_CODE");
        </select>
      </td>
    </tr>
  </table>
  <input type="button" name="edit" value="$editPhrase" onClick="document.prefs.command.value='edit'; if (validateEdit()) document.prefs.submit();">&nbsp;<input type="button" name="delete" value="$deletePhrase" onClick="document.prefs.command.value='delete'; if (validateDelete()) document.prefs.submit();">
END_OF_CODE
  }
  $doc->print(<<"END_OF_CODE");
  &nbsp;<input type="button" name="create" value="$createPhrase" onClick="document.prefs.command.value='create'; document.prefs.submit();">
</form>
</center>
END_OF_CODE

  return $doc
}

sub displayCreateServer
{
  my $self = shift;
  my %args = ( doc => undef, error => "", status => "", @_ );
  my $doc = $args{doc};
  my $error = $args{error};
  my $status = $args{status};
  my $phrases = $self->{langObj}->{phraseObj}->{phrases};
  my $errorPhrase = $phrases->{error};
  my $errStr = "Portal::squidGuard::Server->displayCreateServer()  - $errorPhrase<br>\n";

  my $bgColor = $self->{configObj}->{bgColor};
  my $fgColor = $self->{configObj}->{fgColor};
  my $menufgColor = $self->{configObj}->{menufgColor};
  my $menubgColor = $self->{configObj}->{menubgColor};
  my $cellfgColor = $self->{configObj}->{cellfgColor};
  my $cellbgColor = $self->{configObj}->{cellbgColor};
  my $cellTitlefgColor = $self->{configObj}->{cellTitlefgColor};
  my $cellTitlebgColor = $self->{configObj}->{cellTitlebgColor};
  my $tablebgColor = $self->{configObj}->{tablebgColor};

  my $close = $phrases->{close};
  my $refresh = $phrases->{refresh};
  my $help = $phrases->{help};
  my $mainMenu = $phrases->{serverSettings} . " " . $phrases->{menu};
  my $closeMessage = $phrases->{closeMessage};
  my $serverPhrase = $phrases->{server};
  my $createPhrase = $phrases->{create};
  my $title = sprintf($phrases->{title}, $Portal::squidGuard::VERSION, $createPhrase . " " . $serverPhrase);
  my $selectAllPhrase = $phrases->{selectall};
  my $unselectAllPhrase = $phrases->{unselectall};
  my $blackListsPhrase = $phrases->{blackLists};
  my $passwordPhrase = $phrases->{password};

  $doc->setTitle($title);

  $doc->setFocus("javascript");
  $doc->print(<<"END_OF_CODE");

function showHelp(appName, url, height, width)
{
  var w = window.open(url, appName, "height="+height+",width="+width+",resizable,scrollbars,status");
}

function selectAllGroups()
{
  var bl = document.prefs.blGroups;

  for (var i=0; i < bl.options.length; i++)
  {
    bl.options[i].selected = true;
  }
}

function unselectAllGroups()
{
  var bl = document.prefs.blGroups;

  for (var i=0; i < bl.options.length; i++)
  {
    bl.options[i].selected = false;
  }
}

END_OF_CODE
  $doc->setLinkDecorations();  # turn off underlines on the links.

  my $configObj = Portal::Data::Config->new;
  my $url = ($self->{sessionObj}->{store}->{companyAppObj}->{port} == 80 ? "http" : "https") . "://" . $self->{sessionObj}->{store}->{companyAppObj}->{server} . $configObj->{siteDir} . "/cgi-bin/index.cgi";
  my $mainMenuUrl = $url . "?app=squidGuard&state=Server&command=display";
  my $helpUrl = $url . "?app=Portal&state=Help&command=display&helpApp=squidGuard&content=Normal,Server+Settings";

  $doc->setFocus("body");
  $doc->print(<<"END_OF_CODE");
<center>
<table border="0" width="100%" cellspacing="0" cellpadding="2" bgcolor="$menubgColor">
  <tr>
    <td align="left" width="100%"><a href="$mainMenuUrl" onMouseOver="window.status='$mainMenu'; return true;" title="$mainMenu"><font color="$menufgColor">$mainMenu</font></a> &nbsp;| &nbsp;
    <a href="$helpUrl" onMouseOver="window.status='$help'; return true;" title="$help" onClick="showHelp('help_squidGuard', '$helpUrl', 480, 640); return false;"><font color="$menufgColor">$help</font></a>
    </td>
  </tr>
</table>
END_OF_CODE

  $doc->print("<font color=\"red\">Error</font>: $error<br>\n") if (length $error > 0);
  $doc->print("Status: $status<br>\n") if (length $status > 0);

  # see if we need to lookup the preferences from the database.
  my $serverName = "";
  my $serverPass = "";
  my $blGroups = "";
  my %blGroups = ();

  if (exists $self->{input}->{serverName})
  {
    $serverName = $self->{input}->{serverName};
    $serverPass = $self->{input}->{serverPass};
    $blGroups = $self->{input}->{blGroups};
    foreach my $option (split /\0/, $blGroups)
    {
      $blGroups{$option} = $option;
    }
  }

  # get a list of all Black List groups available in the database.
  my $sth = $self->{dbHandle}->read(sql => "SELECT name FROM groups_tb ORDER BY name");
  if ($self->{dbHandle}->didErrorOccur)
  {
    $self->setError(errorString => $errStr . $self->{dbHandle}->errorMessage);
    return $doc;
  }
  my @groups = ();
  while (my @info = $sth->fetchrow_array)
  {
    $groups[++$#groups] = $info[0];
  }
  $sth->finish;

  my $blOptions = "";

  foreach my $option (@groups)
  {
    $blOptions .= "          <option value=\"$option\"" . (exists $blGroups{$option} ? " selected" : "") . "> $option\n";
  }

  # now generate the form that allows the user to specify their wishes.
  $doc->print(<<"END_OF_CODE");
<br>
<form name="prefs" method="post" action="$url">
  <input type="hidden" name="app" value="squidGuard">
  <input type="hidden" name="state" value="Server">
  <input type="hidden" name="command" value="create">
  <input type="hidden" name="update" value="1">
  <table border="0" cellpadding="2" cellspacing="0" bgColor="$tablebgColor">
    <tr bgColor="$cellTitlebgColor">
      <th align="center"><font color="$cellTitlefgColor">$serverPhrase</font></th>
    </tr>
    <tr>
      <td align="right"><input type="text" name="serverName" value="$serverName"></td>
    </tr>
    <tr>
      <td align="right">$passwordPhrase: <input type="text" name="serverPass" value="$serverPass"></td>
    </tr>
    <tr>
      <td>&nbsp;</td>
    </tr>
    <tr bgcolor="$cellTitlebgColor">
      <th align="center"><font color="$cellTitlefgColor">$blackListsPhrase</font></th>
    </tr>
    <tr>
      <td align="center">
        <select name="blGroups" multiple rows="10">
$blOptions
        </select>
        <br>
        <input type="button" name="selectAll" onClick="selectAllGroups()" value="$selectAllPhrase"><input type="button" name="unselectAll" onClick="unselectAllGroups()" value="$unselectAllPhrase">
      </td>
    </tr>
  </table>
  <input type="submit" name="next" value="$createPhrase">
</form>
</center>
END_OF_CODE

  return $doc
}

sub displayServerPreferences
{
  my $self = shift;
  my %args = ( doc => undef, error => "", status => "", @_ );
  my $doc = $args{doc};
  my $error = $args{error};
  my $status = $args{status};
  my $phrases = $self->{langObj}->{phraseObj}->{phrases};
  my $errorPhrase = $phrases->{error};
  my $errStr = "Portal::squidGuard::Server->displayServerPreferences()  - $errorPhrase<br>\n";

  my $bgColor = $self->{configObj}->{bgColor};
  my $fgColor = $self->{configObj}->{fgColor};
  my $menufgColor = $self->{configObj}->{menufgColor};
  my $menubgColor = $self->{configObj}->{menubgColor};
  my $cellfgColor = $self->{configObj}->{cellfgColor};
  my $cellbgColor = $self->{configObj}->{cellbgColor};
  my $cellTitlefgColor = $self->{configObj}->{cellTitlefgColor};
  my $cellTitlebgColor = $self->{configObj}->{cellTitlebgColor};
  my $tablebgColor = $self->{configObj}->{tablebgColor};

  my $close = $phrases->{close};
  my $refresh = $phrases->{refresh};
  my $help = $phrases->{help};
  my $mainMenu = $phrases->{serverSettings} . " " . $phrases->{menu};
  my $closeMessage = $phrases->{closeMessage};
  my $serverSettingsPhrase = $phrases->{serverSettings};
  my $title = sprintf($phrases->{title}, $Portal::squidGuard::VERSION, $serverSettingsPhrase);
  my $selectAllPhrase = $phrases->{selectall};
  my $unselectAllPhrase = $phrases->{unselectall};
  my $serverRequiredPhrase = $phrases->{serverRequired};
  my $serverInvalidOrNotFoundPhrase = $phrases->{serverInvalidOrNotFound};
  my $serverPhrase = $phrases->{server};
  my $blackListsPhrase = $phrases->{blackLists};
  my $updatePhrase = $phrases->{update};
  my $passwordPhrase = $phrases->{password};

  my $serverID = $self->{input}->{serverID};
  if (length $serverID == 0)
  {
    $self->setError(errorString => $errStr . "$serverRequiredPhrase<br>\n");
    return $doc;
  }
  # verify that this serverID exists.
  my $sth = $self->{dbHandle}->read(sql => "SELECT server FROM servers_tb WHERE id = ? AND cid = ?", plug => [ $serverID, $self->{companyObj}->{id} ]);
  if ($self->{dbHandle}->didErrorOccur)
  {
    $self->setError(errorString => $errStr . $self->{dbHandle}->errorMessage);
    return $doc;
  }
  my @info = $sth->fetchrow_array;
  if (not defined $info[0])
  {
    $self->setError(errorString => $errStr . sprintf($serverInvalidOrNotFoundPhrase, $serverID) . "<br>\n");
    return $doc;
  }
  my $oldServerName = $info[0];
  $sth->finish;

  $doc->setTitle($title);

  $doc->setFocus("javascript");
  $doc->print(<<"END_OF_CODE");

function showHelp(appName, url, height, width)
{
  var w = window.open(url, appName, "height="+height+",width="+width+",resizable,scrollbars,status");
}

function selectAllGroups()
{
  var bl = document.prefs.blGroups;

  for (var i=0; i < bl.options.length; i++)
  {
    bl.options[i].selected = true;
  }
}

function unselectAllGroups()
{
  var bl = document.prefs.blGroups;

  for (var i=0; i < bl.options.length; i++)
  {
    bl.options[i].selected = false;
  }
}

END_OF_CODE
  $doc->setLinkDecorations();  # turn off underlines on the links.

  my $configObj = Portal::Data::Config->new;
  my $url = ($self->{sessionObj}->{store}->{companyAppObj}->{port} == 80 ? "http" : "https") . "://" . $self->{sessionObj}->{store}->{companyAppObj}->{server} . $configObj->{siteDir} . "/cgi-bin/index.cgi";
  my $mainMenuUrl = $url . "?app=squidGuard&state=Server&command=display";
  my $refreshUrl = $url . "?app=squidGuard&state=Server&command=edit&serverID=$serverID";
  my $helpUrl = $url . "?app=Portal&state=Help&command=display&helpApp=squidGuard&content=Normal,Server+Settings";

  $doc->setFocus("body");
  $doc->print(<<"END_OF_CODE");
<center>
<table border="0" width="100%" cellspacing="0" cellpadding="2" bgcolor="$menubgColor">
  <tr>
    <td align="left" width="100%"><a href="$mainMenuUrl" onMouseOver="window.status='$mainMenu'; return true;" title="$mainMenu"><font color="$menufgColor">$mainMenu</font></a> &nbsp;| &nbsp;
    <a href="$helpUrl" onMouseOver="window.status='$help'; return true;" title="$help" onClick="showHelp('help_squidGuard', '$helpUrl', 480, 640); return false;"><font color="$menufgColor">$help</font></a> &nbsp;| &nbsp;
    <a href="$refreshUrl" onMouseOver="window.status='$refresh'; return true;" title="$refresh"><font color="$menufgColor">$refresh</font></a></td>
  </tr>
</table>
END_OF_CODE

  $doc->print("<font color=\"red\">Error</font>: $error<br>\n") if (length $error > 0);
  $doc->print("Status: $status<br>\n") if (length $status > 0);

  # see if we need to lookup the preferences from the database.
  my $serverName = "";
  my $serverPass = "";
  my $blGroups = "";
  my %blGroups = ();

  if (exists $self->{input}->{serverName})
  {
    $serverName = $self->{input}->{serverName};
    $serverPass = $self->{input}->{serverPass};
    $blGroups = $self->{input}->{blGroups};
    foreach my $option (split /\0/, $blGroups)
    {
      $blGroups{$option} = $option;
    }
  }
  else # goto the database.
  {
    my $sth = $self->{dbHandle}->read(sql => "SELECT server, bl_groups, passwd FROM servers_tb WHERE id = ?", plug => [ $serverID ]);
    if ($self->{dbHandle}->didErrorOccur)
    {
      $self->setError(errorString => $errStr . $self->{dbHandle}->errorMessage);
      return $doc;
    }
    my @info = $sth->fetchrow_array;
    $sth->finish;
    $serverName = $info[0];
    $blGroups = $info[1];
    $serverPass = $info[2];
    foreach my $option (split /\|/, $blGroups)
    {
      $blGroups{$option} = $option;
    }
  }

  # get a list of all Black List groups available in the database.
  my $sth = $self->{dbHandle}->read(sql => "SELECT name FROM groups_tb ORDER BY name");
  if ($self->{dbHandle}->didErrorOccur)
  {
    $self->setError(errorString => $errStr . $self->{dbHandle}->errorMessage);
    return $doc;
  }
  my @groups = ();
  while (my @info = $sth->fetchrow_array)
  {
    $groups[++$#groups] = $info[0];
  }
  $sth->finish;

  my $blOptions = "";

  foreach my $option (@groups)
  {
    $blOptions .= "          <option value=\"$option\"" . (exists $blGroups{$option} ? " selected" : "") . "> $option\n";
  }

  # now generate the form that allows the user to specify their wishes.
  $doc->print(<<"END_OF_CODE");
<br>
<form name="prefs" method="post" action="$url">
  <input type="hidden" name="app" value="squidGuard">
  <input type="hidden" name="state" value="Server">
  <input type="hidden" name="command" value="edit">
  <input type="hidden" name="update" value="1">
  <input type="hidden" name="oldServerName" value="$oldServerName">
  <input type="hidden" name="serverID" value="$serverID">
  <table border="0" cellpadding="2" cellspacing="0" bgColor="$tablebgColor">
    <tr bgColor="$cellTitlebgColor">
      <th align="center"><font color="$cellTitlefgColor">$serverPhrase</font></th>
    </tr>
    <tr>
      <td align="right"><input type="text" name="serverName" value="$serverName"></td>
    </tr>
    <tr>
      <td align="right">$passwordPhrase: <input type="text" name="serverPass" value="$serverPass"></td>
    </tr>
    <tr>
      <td>&nbsp;</td>
    </tr>
    <tr bgcolor="$cellTitlebgColor">
      <th align="center"><font color="$cellTitlefgColor">$blackListsPhrase</font></th>
    </tr>
    <tr>
      <td align="center">
        <select name="blGroups" multiple rows="10">
$blOptions
        </select>
        <br>
        <input type="button" name="selectAll" onClick="selectAllGroups()" value="$selectAllPhrase"><input type="button" name="unselectAll" onClick="unselectAllGroups()" value="$unselectAllPhrase">
      </td>
    </tr>
  </table>
  <input type="submit" name="next" value="$updatePhrase">
</form>
</center>
END_OF_CODE

  return $doc
}

1;
__END__

=head1 NAME

Server - Object used to build a Server Object Class.

=head1 SYNOPSIS

  use Portal::squidGuard::Server;
  my $obj = Portal::squidGuard::Server->new;
  if ($obj->didErrorOccur())
  {
    die $obj->errorMessage();
  }
  $obj->run(command => $command);

=head1 DESCRIPTION

Server is a Server class.

=head1 Exported FUNCTIONS

  scalar new(langObj, input, cookieObj, dbHandle, logObj, sessionObj,
             userObj, companyObj, applicationObj, appObj, portalSession,
             browserType, browserVersion, browserMaker, authObj)
    Creates a new instance of the Portal::squidGuard::Server
    object.

  bool isValid(void)
    Returns 0 or 1 to indicate if the object is valid.  The error will 
    be available via errorMessage().
    
  void setError(errorString)
    Sets error = 1 and errorString = errorString.

  scalar didErrorOccur(void)
    Returns the value of error.

  scalar errorMessage(void)
    Returns the value of errorString.
    
  HTMLObject run(command)
    Runs the specified command and returns the resulting HTMLObject.


  NOTE:  All data fields are accessible by specifying the object
         and pointing to the data member to be modified on the
         left-hand side of the assignment.
         Ex.  $obj->variable($newValue); or $value = $obj->variable;

=head1 AUTHOR

PC & Web Xperience, Inc. (mailto:admin@pcxperience.com)

=head1 SEE ALSO

perl(1), Portal(3), Portal::squidGuard(3)

=cut
